/*
 * grv.c
 *
 * Main program - converted from BASIC
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include "highscore.h"
#include "graphics.h"
#include "grv.h"

struct gameparams gp;
struct monster ghost[MAXGHOST];
#define DefEscStart 15
int EscStart = DefEscStart;
struct customized custom;

static void init_gameparams(void)
{
    gp.Sc = gp.SBs = 0;
    gp.Life = min((unsigned int)opt.life, 6);
    gp.InfLife = opt.life < 0;
    gp.SBs = 0;
    gp.Lvf = opt.hyperspace ? 4 : 0;
    gp.GkCh = min((unsigned int)opt.power, 9999);
    gp.KulSpr = min((unsigned int)opt.kulspr, 99);
    gp.MLev = gp.Level = min(opt.startlevel, ELev - 1);
    gp.Cheat = opt.cheat || opt.startlevel || opt.hyperspace ||
	opt.kulspr || opt.power || (opt.life != 3);
    gp.EOLWait = 1;

    gp.gameid = genrand_uint64(); /* Better than nothing */
    gp.have_id = !get_random_bytes(&gp.gameid, sizeof gp.gameid);

    if (EscStart >= ELev)
	EscStart = ELev - 1;
    else if (EscStart < 0)
	EscStart = DefEscStart;
}

void initscreen(int w)
{
    if (opt.debug) {
	fprintf(stderr, "initscreen(%d), gp.c = %d\n", w, gp.c);
    }

    width(w);
    color(0, gp.c);
    cls();
}

static void newlevel(void)
{
    gp.Bon = 0;
    gp.TF = -1;
    gp.Hyp = 0;
    gp.Bar = 0;
    gp.ChBd = gp.GkCh;
    gp.FS = 1;			/* Ghosts are not frozen or reversed */
    gp.FScnt = gp.FSdiv = 1;	/* Ghosts are at full speed */
    /* 150 ms/step is the speed for level 0 */
    gp.Speed = (150 * ELev) / (gp.Level + ELev);
    gp.Tid = 0.0;
    if (gp.Level < ELev - 1)
	gp.ZLevel = ELev - 1;

    if (gp.Level >= EscStart && gp.MLev < EscStart) {
	/* Announce Escape option */
	levelscreen();
	color(0, gp.c2);
	lputcn(10, 1, ' ', 40);
	lprintf(11, 1, " Now you have completed the first %-6d", EscStart);
	lprint(12, 1, " levels of this game.  Now you can use  ");
	lprint(13, 1, " your escape key to get away!           ");
	lputcn(14, 1, ' ', 40);
	lprint(15, 1, " Your escape key is: ");
	color(14, gp.c2);
	gprintf("%-19.19s", SDL_GetKeyName(kbd_keys.esc));
	lputcn(16, 1, ' ', 40);

	if (gp.EOLWait)
	    mymssleep(5000);
    }

    gp.EOLWait = 1;

    gp.MLev = max(gp.Level, gp.MLev);

    reset_actions();
    addaction(0, 0, 1800.0, 0, act_toolate);	/* 30 minutes -> dead */

    if (gp.Level == ELev - 1) {
	/* Last level */
	gp.KulSpr = 0;
    } else {
	/* Not the last level */
	int e;
	for (e = gp.KulSpr; e; e--) {
	    if (irnd(10))
		gp.KulSpr--;	/* 10% chance to keep unused shots */
	}
	if (gp.KulSpr > 80)
	    gp.KulSpr = 80;
    }

    if (gp.Level > gp.MLev)
	gp.MLev = gp.Level;

    drawlevel();

    play();
}

static void game(void)
{
    init_gameparams();
    while (gp.Level < ELev && gp.Life >= 0) {
	newlevel();
    }
    if (!(opt.once && gp.Cheat)) {
	highscore_endgame();
    }
}

struct opts opt;

int main(int argc, char *argv[])
{
    int i;

    randomize();

    set_levels(MAX_LEVEL);	/* Default level count */

    memset(&opt, 0, sizeof opt);
    opt.life = 3;

    for (i = 1; i < argc; i++) {
	const char *o = argv[i];
	if (*o != '-') {
	    int err = load_game_script(o);
	    if (err) {
		fprintf(stderr, "%s: failed to load game script %s: %s\n",
			argv[0], o, strerror(err));
		exit(1);
	    }
	    continue;
	}
	o++;
	if (*o == '-')		/* Support -foo and --foo */
	    o++;

	if (!o)			/* End of list marker */
	    continue;

	if (!strcmp(o, "nonet") || !strcmp(o, "no-net"))
	    opt.nonet = true;
	else if (!strcmp(o, "nohighscore") || !strcmp(o, "no-high-score"))
	    opt.nohighscore = true;
	else if (!strcmp(o, "editor")) {
	    opt.editor = true;
	    opt.nohighscore = true;
	    opt.nonet = true;
	    opt.cheat = true;
	    opt.start = true;
	    opt.once = true;
	} else if (!strncmp(o, "window", 6)) {
	    o += 6;
	    opt.window = 1;
	    if (*o == '=') {
		opt.window = atoi(o + 1);
	    } else {
		while (*o++ == '+')
		    opt.window++;
	    }
	} else if (!strcmp(o, "debug")) {
	    opt.debug = true;
	} else if (!strcmp(o, "start")) {
	    opt.start = true;
	} else if (!strcmp(o, "once")) {
	    opt.once = true;
	} else if (!strcmp(o, "test") || !strcmp(o, "cheat")) {
	    if (!strcmp(o, "cheat"))
		opt.cheat = true;
	    while ((o = argv[i + 1]) && *o && *o != '-') {
		if (!strcmp(o, "maxlife")) {
		    opt.life = 6;
		} else if (!strcmp(o, "inflife")) {
		    opt.life = -1;
		} else if (!strncmp(o, "lives=", 6) || !strncmp(o, "life=", 5)) {
		    opt.life = strtoul(strchr(o, '=') + 1, NULL, 10);
		} else if (!strcmp(o, "hyper")) {
		    opt.hyperspace = true;
		} else if (!strncmp(o, "shots=", 6)) {
		    opt.kulspr = strtoul(o + 6, NULL, 10);
		} else if (!strncmp(o, "power=", 6)) {
		    opt.power = strtoul(o + 6, NULL, 10);
		} else if (!strncmp(o, "level=", 6)) {
		    opt.startlevel = strtoul(o + 6, NULL, 10) - 1;
		    if (opt.startlevel < 0)
			opt.startlevel = 0;
		} else if (!strncmp(o, "endlevel=", 9)) {
		    opt.endlevel = strtoul(o + 9, NULL, 10);
		    set_levels(opt.endlevel);
		} else {
		    fprintf(stderr, "%s: unknown cheat option: %s (ignoring)\n",
			    argv[0], o);
		}
		i++;
	    }
	} else if (!strcmp(o, "map") || !strcmp(o, "maps")) {
	    /* Followed by a list of level maps to load */
	    while ((o = argv[i + 1]) && *o && *o != '-') {
		int err = load_map_file(o, -1, -1);
		if (err) {
		    fprintf(stderr, "%s: failed to load map %s: %s\n",
			    argv[0], o, strerror(err));
		    exit(1);
		}
		i++;
	    }
	} else if (!strcmp(o, "game")) {
	    /* Followed by a list of game scripts to load */
	    while ((o = argv[i + 1]) && *o && *o != '-') {
		int err = load_game_script(o);
		if (err) {
		    fprintf(stderr, "%s: failed to load game script %s: %s\n",
			    argv[0], o, strerror(err));
		    exit(1);
		}
		i++;
	    }
	} else {
	    fprintf(stderr, "%s: unknown option: %s\n", argv[0], argv[i]);
	    exit(1);
	}
    }

    /* Init the highscore data structures */
    /* This is done here because it's a destructive operation and must
       only be done once! */
    highscore_init();

    load_prefs();

    if (screen_init(opt.window)) {
	fprintf(stderr, "%s: Cannot initalize graphics!\n", argv[0]);
	exit(1);
    }

    if (!opt.nohighscore) {
	highscore_load();
	highscore_download();
	highscore_upload(); /* If we have something the server didn't... */
    }

    drawlevel_init();		/* Game engine one-time initialization */

    if (opt.start) {
	game();
	if (opt.once)
	    return 0;
    }

    while (1) {
	intro();
    again:
	switch (get_key()->keysym.sym) {
	case SDLK_k:
	    edit_keys();
	    break;
	case SDLK_p:
	    game();
	    if (opt.once)
		return 0;
	    break;
	case SDLK_q:
	    return 0;
	case SDLK_v:
	    highscore_show(0, 0);
	    break;
	default:
	    goto again;		/* Still on the intro screen, don't redraw */
	}
    }
}
