#ifndef GRV_H
#define GRV_H 1

#include "compiler.h"
#include "graphics.h"
#include "grvfiledata.h"

#include <SDL_keyboard.h>

typedef struct grvfilerandom gfr;
typedef struct grvfileitem gfi;

#ifndef min
# define min(x,y) ((x)<(y)?(x):(y))
#endif
#ifndef max
# define max(x,y) ((x)>(y)?(x):(y))
#endif

/* Get random bytes from the system */
int get_random_bytes(void *, int);
void randomize(void);

/* Internal pseudo-random number generator */
double genrand_res53(void);	/* 0.0 <= x < 1.0 */
#define rnd() genrand_res53()
uint32_t genrand_uint32(void);
uint64_t genrand_uint64(void);
void init_by_array(const unsigned long *, unsigned long);
uint32_t irnd(uint32_t n);	/* 0 <= x < n */

/* Command-line options */
struct opts {
    bool nonet;			/* Don't access the network server */
    bool nohighscore;		/* No highscore handling at all */
    bool editor;		/* Playtesting from editor (placeholder) */
    bool debug;			/* Log debug output to stderr */
    bool start;			/* Start immediately, no menu */
    bool once;			/* Quit after one game */
    bool cheat;			/* Force the cheat flag on for every game */
    bool hyperspace;		/* Start with level hyperspace cheat */
    int window;			/* Run in a window/scaling factor */
    int startlevel;		/* Start at a specific level cheat */
    int endlevel;		/* Move the end level (1-based!) */
    int life;			/* Override default lives (-1 for infinite) */
    unsigned int kulspr;	/* Start with shots cheat */
    unsigned int power;		/* Start with power cheat */
};
extern struct opts opt;

extern int ELev;		/* Number of levels total */
extern int EscStart;		/* First level with Escape (-1) */

struct xy {
    uint8_t x, y;
};

struct gameparams {
    /* Yes, score can be negative, although it's highly unlikely */
    int64_t Sc;			/* Score */
    int64_t StScore;		/* Score at start of level */
    int64_t SBs;		/* Total points in Super Bonus */
    int Life;			/* Lives left */
    int Lvf;
    int GkCh;
    int Level;			/* Current level */
    int MLev;			/* Max level achieved */
    int ZLevel;
    int KulSpr;
    int Bon;			/* Bonus available right now */
    int TF;			/* double? */
    int FS;			/* Ghost direction (1 normal, -1 scared, 0 frozen) */
    int FSdiv;			/* Ghost movement divider */
    int FScnt;			/* Divider down counter */

    int x, y;			/* Player coordinates */
    int XWk, YWk;		/* Player movement */
    int Hyp;
    int ChBd;
    int Bar;			/* Number of cherries on screen */
    int OrigBar;		/* Cherries at level start */
    int Bombs;			/* Number of bombs on the level */
    union {
	struct {
	    /* For historical reasons */
	    int c;		/* Main level color */
	    int c2;		/* Alternate level color */
	};
	int cx[2];		/* As an array */
    };
    int Cheat;			/* Cheat codes used */
    int InfLife;		/* Infinite life cheat */
    int EOLWait;		/* Standard death/end of level pause? */
    int TZero;			/* Time at start of level (ms) */
    int Speed;			/* Desired game speed ms/game round */
    double Tid;			/* Time since level start (float s) */
    enum {
	Status_Live,		/* Player alive */
	Status_Dead,		/* Player dead */
	Status_Done,		/* Player done with level */
	Status_Quit,		/* Level exit (treasure) */
    } Status;

    int nwhite;			/* Number of white cherries */
    int nextwhite;		/* Next white cherry (if any) */
    struct xy whitecherrylist[190];

    uint64_t gameid;		/* (Hopefully) unique ID for high score list */
    int have_id;		/* Is ID frozen yet? */
};
extern struct gameparams gp;

enum actions {
    act_none,			/* No action */
    act_apple,			/* Apple fall */
    act_bomb,			/* Bomb goes off */
    act_bonus_on,		/* BONUS light on */
    act_bonus_off,		/* BONUS light off */
    act_smash,			/* "Smash" */
    act_toolate,		/* Time expired */
    act_droplevel,		/* Drop one level (finale level) */
    act_door,			/* Open door */
    act_ghost_thaw,		/* Thaw frozen ghosts */
    act_ghost_normal,		/* Return ghosts to normal */
};

struct monster {
    int x, y;
    int dead;
};
#define MAXGHOST 5
extern struct monster ghost[MAXGHOST];

/* Keyboard handling */
#define NKEYS 13

struct kbd_direct {
    SDL_Keycode up;
    SDL_Keycode left;
    SDL_Keycode right;
    SDL_Keycode down;
};
union kbd_keys {
    struct {
	struct kbd_direct move;		/* Move directions */
	struct kbd_direct shoot;	/* Shoot directions */
	SDL_Keycode stop;		/* Stop */
	SDL_Keycode esc;		/* Escape (teleport) */
	SDL_Keycode pause;		/* Pause game */
	SDL_Keycode fullscreen;		/* Toggle full screen */
	SDL_Keycode screenshot;		/* Capture a screen shot */
    };
    SDL_Keycode keys[NKEYS];
};
extern union kbd_keys kbd_keys;

/* User-defined SDL event types */
enum user_event {
    event_next_round,		/* Next game round */
    event_blink,		/* Handle blinking */
    event_sleep			/* Sleep expired */
};

/* Character symbols */
#define SYM_PLAYER	0x01
#define SYM_GHOST	0x02
#define SYM_ROCK	0x04
#define SYM_CLUSTER	0x0f
#define SYM_NOTHING	' '
#define SYM_DIAMOND	'*'
#define SYM_MYSTERY	'?'
#define SYM_POROUS_WALL	0xb0
#define SYM_WALL	0xb1
#define SYM_CHERRY	0xeb
#define SYM_BOMB	0xe5
#define SYM_SHOT	0xec
#define SYM_BONUS	0xf9
#define SYM_APPLE	0xfe
/* All possible doors */
#define DOORS		"\xb9\xba\xbb\xbc\xc8\xc9\xca\xcb\xcc\xcd\xce"
#define HYPER		"HYPER"

#define case_door	case 0xb9: case 0xba: case 0xbb: case 0xbc: \
    case 0xc8: case 0xc9: case 0xca: case 0xcb: case 0xcc: case 0xcd: \
    case 0xce
#define case_hyper	case 'H': case 'Y': case 'P': case 'E': case 'R'

/* utils.c */
typedef SDL_TimerID timer_id;
uint32_t post_periodic(uint32_t interval, void *param);
uint32_t post_oneshot(uint32_t interval, void *param);
void mymssleep(int ms);
char *format_time(double t);
int wait_event(SDL_Event *event);
timer_id timer_oneshot(uint32_t ms, enum user_event event);
timer_id timer_periodic(uint32_t ms, enum user_event event);
#define timer_cancel(x) SDL_RemoveTimer(x)

/* drawlevel.c */
#define BSIZE (22*40)
extern struct xy wholeboard[BSIZE];
void drawlevel(void);
void levelscreen(void);
void drawlevel_init(void);
int calc_random(const gfr * rnd);
double calc_randomf(const gfr * rnd);
void drawfixeditem(int r, int c, const gfi * what, const gfr * tmr);
void drawrandomitems(const gfr * cnt, const gfr * tmr);

/* loadmap.c */
bool load_map_data(const struct grvzfileheader *zhdr, const void *zdata,
		   int baselevel, int endlevel);
bool draw_data_level(void);
int load_map_file(const char *filename, int baselevel, int endlevel);
void set_levels(int count);
int load_game_script(const char *filename);

/* grv.c */
struct customized {
    const char *title;
    const char *byline;
    const char *author;
};
extern struct customized custom;
void initscreen(int w);
void update_score(void);
void update_shots(void);
void update_power(void);
void update_jump(void);
void message(int col, const char *fmt, ...);

/* play.c */
void white2black(void);
void play(void);
void fall_rock(int x, int y);
void fall_apple(int x, int y);
void kill_ghost(int x, int y);
void take_cherry(void);
void take_diamond(void);
void check_activation(int r, int c);

/* keyboard.c */
void push_key(SDL_KeyboardEvent * ke);
SDL_KeyboardEvent *poll_key(void);
SDL_KeyboardEvent *get_key(void);
bool immediate_event(SDL_Event *event);

/* action.c */
void reset_actions(void);
void addaction(int x, int y, double when, unsigned int priority,
	       enum actions what);
void removeaction(int x, int y, enum actions what);
void handle_action(void);
void retime_all(double when, enum actions what);

/* bullets.c */
void add_bullet(int x, int y, int dx, int dy, int color);
void run_bullets(void);

/* mystery.c */
void mystery(void);

/* highscore.c */
void highscore_load(void);
void highscore_download(void);
void highscore_save(void);
void highscore_upload(void);
void highscore_show(int myrank, int save);
void highscore_endgame(void);

/* intro.c */
void intro(void);

/* prefs.c */
void load_prefs(void);
void edit_keys(void);

/* configdir.c */
const char *app_config_dir(const char *set_dir);
const char *app_global_etcdir(void);
const char *app_global_datadir(void);
char *app_config_file(const char *filename, bool for_write);
FILE *fopen_config(const char *file, const char *mode);

#endif				/* GRV_H */
