/* -*- c -*-
 *
 * This is a C and C++ shared header file
 */

#ifndef GRVFILEDATA_H
#define GRVFILEDATA_H

#include <inttypes.h>

/* These are on disk structures; padding is not OK */
#pragma GCC diagnostic push
#pragma GCC diagnostic error "-Wpadded"

struct grvfileheader {
    char magic[8];
    uint32_t major;
    uint32_t incompatflags;
    uint32_t rocompatflags;
    uint32_t compatflags;
    uint32_t headersize;

    /* File offsets of tables */
    uint32_t boardoffset;
    uint32_t randoffset;

    /* Object counts */
    uint8_t xsize, ysize, randobjs, timerbits;

    uint16_t baselevel;		/* Intended base level */
    uint16_t resv1;

    uint8_t usedtimers;		/* Maximum *used* timer index + 1 */
    uint8_t bombtimer;		/* Default bomb timer */
    uint8_t doortimer;		/* Default door timer */
    uint8_t resv2;

    uint32_t resv3[4];
};

/* Bigendian due to qCompress - qCompress produces the dsize field */
struct grvzfileheader {
    char magic[8];
    uint32_t zsize;		/* BIGENDIAN compressed size */
    uint32_t dsize;		/* BIGENDIAN uncompressed size */
};

#define GRVFILE_MAGIC "#!grv\n\xff"	/* + final implicit NUL */
#define GRVDATA_MAGIC "#!grv\n\xfd"	/* + final implicit NUL */
#define GRVDATA_MAJOR 1
#define GRV_MAX_DATA	(1024*1024)	/* Safety check */

struct grvfileitem {
    uint8_t itemtype;
    uint8_t itemflags;
    uint16_t timerprio;
};

enum grvfileitemflags {
    GFI_BG_MASK = 0x01,
    GFI_TUNNEL = 0x80
};

struct grvfilerandom {
    uint8_t itemtype;		/* Timer index for a timer */
    uint8_t rndflags;
    uint16_t minlvl;
    int32_t valmax;

    double valbase;
    double vallvl;
    double valrnd;
};

enum grvfilerandomflags {
    GFR_ITEM = 0x00,		/* A plain item */
    GFR_TIMER = 0x01,		/* It is a timer */
    GFR_MULTI = 0x02		/* Multi-randomness */
};

#pragma GCC diagnostic pop

#endif				/* GRVFILEDATA_H */
