#include <time.h>
#include "grv.h"

#ifdef __unix__
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

/*
 * At least try to close-on-exec().
 * Otherwise, well, at least it will be closed on exit().
 */
#ifndef O_CLOEXEC
# define O_CLOEXEC 0
#endif

static int xread(int fd, void *buf, size_t count)
{
    char *b = (char *)buf;
    int n = 0;
    int rv;

    while (count) {
	rv = read(fd, b, count);
	if (rv == -1) {
	    if (errno == EINTR || errno == EAGAIN)
		continue;
	    else
		return n ? n : -1;
	} else if (rv == 0) {
	    return n;
	}

	count -= rv;
	n += rv;
	b += rv;
    }
    return n;
}

static int randfd = -1;

int get_random_bytes(void *ptr, int len)
{
    /* We might have /dev/urandom */
    if (randfd < 0) {
	randfd = open("/dev/urandom", O_RDONLY|O_CLOEXEC);

	if (randfd < 0)
	    return -1;
    }

    return xread(randfd, ptr, len) == len ? 0 : -1;
}

#elif defined(_WIN32)

#include <windows.h>
#include <bcrypt.h>

int get_random_bytes(void *ptr, int len)
{
    return -!!BCryptGenRandom(NULL, ptr, len, BCRYPT_USE_SYSTEM_PREFERRED_RNG);
}

#else

/* Need system-specific code here */

int get_random_bytes(void *ptr, int len)
{
    return -1;
}

#endif

void randomize(void)
{
    unsigned long key_array[32 / sizeof(unsigned long)];

    key_array[0] = time(0);	/* Better than nothing on failure */

    get_random_bytes(key_array, sizeof key_array);

    init_by_array(key_array, sizeof key_array / (sizeof key_array[0]));
}
